from django.db import models
from jinja2 import Template as TemplateClass
from jinja2.environment import Environment

class Template(models.Model):
    '''
    Stores a Django/Jinja template into the database for use with
    quartet-capture/number range, etc, ...
    The render method will return a string with the template and variables
    updated with the context dictionary.
    '''
    name = models.CharField(
        max_length=100,
        null=False,
        blank=False,
        db_index=True,
        unique=True,
        help_text="A name to identify this template")
    content = models.TextField(
        null=False,
        blank=False,
        help_text="The full Django/Jinja template to be used.")
    description = models.CharField(
        max_length=200,
        null=True,
        blank=True)

    def render(self, context, environment: Environment = None):
        '''
        Renders the template passing a dictionary of key/value pairs
        in the context parameter.
        '''
        if not environment:
            environment = Environment(
                trim_blocks=True,
                lstrip_blocks=True,
                autoescape=True
            )
        template = environment.from_string(self.content)
        return template.render(context)



# Generated by Django 2.1.1 on 2018-09-27 17:04

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, help_text='A name to identify this template', max_length=100, unique=True)),
                ('content', models.TextField(help_text='The full Django/Jinja template to be used.')),
                ('description', models.CharField(blank=True, max_length=200, null=True)),
            ],
        ),
    ]



# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
# Copyright 2018 SerialLab Corp.  All rights reserved.

import sys
import random
from uuid import uuid4
from time import time
from datetime import datetime

from jinja2.environment import Environment
from quartet_templates.models import Template
from quartet_capture.rules import Step, RuleContext


class TemplateStep(Step):
    """
    Will load a template based on the step parameter "Template Name" and
    render it using a combination of rule data, context variables and
    the template variables.
    """

    def execute(self, data, rule_context: RuleContext):
        self.info('Looking for Step Parameter with name "Template Name"...')
        template_name = self.get_parameter(
            'Template Name',
            raise_exception=True
        )
        context_key = self.get_parameter("Context Key")
        self.info("Template Name parameter value found: %s.  "
                  "Looking up Template...", template_name)
        template = Template.objects.get(name=template_name)
        self.info("Found template...rendering...")
        environment = Environment(
            trim_blocks=True,
            lstrip_blocks=True,
            autoescape=self.get_boolean_parameter('Auto Escape', True)
        )
        ret = template.render({'data': data, 'rule_context': rule_context,
                               'step_parameters': self.parameters,
                               'task_parameters': self.get_task_parameters(
                                   rule_context
                               ),
                               'epoch': time(),
                               'random': random.randint(1, sys.maxsize),
                               'UUID': str(uuid4()),
                               'datetime': datetime.isoformat(datetime.now())
                               }, environment=environment)
        self.info("Template response summary %s:", ret[:10000])
        if context_key:
            self.info("Placing rendered content into context key %s.",
                      context_key)
            rule_context.context[context_key] = ret
        else:
            self.info("Returning rendered content to the rule.")
            data = ret
        return data

    @property
    def declared_parameters(self):
        return {
            "Template Name": "The name of the template to load and render.",
            "Context Key": "The context key to place the output into.  If "
                           "none is specified, the output will be returned to "
                           "the rule in place of the inbound data.",
            "Auto Escape": "Whether or not the jinja2 template should be "
                           "auto escaped.  Default is True, set to False if "
                           "you are embedding XML within XML, etc."
        }

    def on_failure(self):
        pass



import os
from django.test import TestCase
from quartet_templates import models
from quartet_capture.models import Rule, Step, Task, StepParameter
from quartet_capture.rules import Rule as CRule

class TemplateTest(TestCase):
    '''
    Tests the Template model.
    '''
    def create_template(self, name: str, content: str, description: str):
        return models.Template.objects.create(name=name, content=content, description=description)

    def test_template_creation(self):
        content = '''
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:urn="urn:bla:bla">
   <soapenv:Header/>
   <soapenv:Body>
      <urn:SpecialXML>
         <SomethingSpecial>{{ something_special }}</SomethingSpecial>
      </urn:SpecialXML>
   </soapenv:Body>
</soapenv:Envelope>
        '''
        template = self.create_template(name="Test Template", content=content, description="A Test Template")
        self.assertTrue(isinstance(template, models.Template))

    def test_template_rendering(self):
        content = '''
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:urn="urn:bla:bla">
   <soapenv:Header/>
   <soapenv:Body>
      <urn:SpecialXML>
         <SomethingSpecial>{{ something_special }}</SomethingSpecial>
      </urn:SpecialXML>
   </soapenv:Body>
</soapenv:Envelope>
        '''
        template = self.create_template(name="Test Template", content=content, description="A Test Template")
        rendered = template.render(context={"something_special": "Special Value"})
        expected = '''
<soapenv:Envelope xmlns:soapenv="http://schemas.xmlsoap.org/soap/envelope/" xmlns:urn="urn:bla:bla">
   <soapenv:Header/>
   <soapenv:Body>
      <urn:SpecialXML>
         <SomethingSpecial>Special Value</SomethingSpecial>
      </urn:SpecialXML>
   </soapenv:Body>
</soapenv:Envelope>
        '''
        self.assertEqual(expected, rendered)

    def test_steps(self):
        vals = range(1000000,1000100)
        self._create_test_template()
        db_rule, db_task, db_step = self._create_rule()
        c_rule = CRule(db_task.rule, db_task)
        c_rule.execute(vals)
        print(c_rule.data)
        
    def _create_rule(self):
        db_rule = Rule()
        db_rule.name = 'Test Rule'
        db_rule.description = 'Template test rule.'
        db_rule.save()

        # create a new step
        db_step = Step()
        db_step.name = 'handle template'
        db_step.description = 'Template test.'
        db_step.order = 1
        db_step.step_class = 'quartet_templates.steps.TemplateStep'
        db_step.rule = db_rule
        db_step.save()
        sp = StepParameter.objects.create(
            step=db_step,
            name="Template Name",
            value="Test Template"
        )
        db_task = Task(
            rule=db_rule,
            status='QUEUED',
        )
        return db_rule, db_task, db_step

    def _create_test_template(self):
        models.Template.objects.create(
            name="Test Template",
            description="A test template",
            content=self._get_file_data()
        )

    def _get_file_data(self):
        file_path = '../tests/data/test_template.xml'
        curpath = os.path.dirname(__file__)
        f = open(os.path.join(curpath, file_path))
        return f.read()